
/*  Colorimétrie : Mesure de couleur RVB
Mesures cumulées
Correction selon feuilles de données
Normalisation à 255
*/

// Gestion des entrées du capteur de couleurs SparkFun Color Sensor Breakout - HDJD-S822
// 0 Bleu entrée sur A0, 1 Vert entrée sur A1, 2 Rouge entrée sur A2 
#define ledPin 5  // Commande LED sur D5  Led blanche de test

// Calcul des correction sur la base des sensibilité relatives (graphe p7)
// Bleu 460 nm  Sb=0.48 48%       Vert 542 nm Sg=0.75        Rouge 645 nm  Sr=0.85
const float CoefCorrection[3]={0.48 , 0.75, 0.85}

int ColorRdg[3];// triplet lecture brute 
float ColorC[3]; // triplet corrigé selon feuilles de données p7 du capteur
float ColorN[3]; // triplet corrigé normalisé

float ColorMax;  
// -----------------------------------

void setup() 
{
  Serial.begin(9600);
  analogReference(EXTERNAL);// AREF relié à 3.3V sur le shield (donc 1023 correspond à 3.3V)
  pinMode(ledPin, OUTPUT);
}// fin de setup

void loop(void) 
{
  // Initialisation d'un cycle de lecture des trois tensions
  ColorRdg[0]=ColorRdg[1]=ColorRdg[2]=0; 
  digitalWrite(ledPin, HIGH);  // éclairage de l'objet coloré avec la led blanche
  delay(500);
  
  byte n=0; byte i;
  while (n<10) { // cumul de 10 lectures successives pour effet de lissage. 
    for (i=0; i<3;i++) {      // Le calcul de la moyenne par division n'est pas indispensable (cf. normalisation)
        ColorRdg[i] = analogRead(i) + ColorRdg[i] ;
        }
    //delay(100);    
    n=n+1;
  } // Fin des 3*10 lectures

  digitalWrite(ledPin, LOW);// Fin de la mesure donc Eteindre la LED 
  
  // Correction d la différence de sensibilité spectrale des trois photodiodes
  for (i=0; i<3;i++) {
        ColorC[i] = ColorRdg[i]/CoefCorrection[3]; // reel = entier/reel
  }
  
  // Recherche de la valeur max dans le triplet corrigé  (au moins 6 lignes avec des if)
  ColorMax = max(max(ColorC[0],ColorC[1]),ColorC[2]);
  
  // Normalisation à 255 pour chaque triplé mesuré : au moins l'une des trois coordonnées de couleur sera à 255
  for (i=0; i<3;i++) {
        ColorN[i] = 255.0*ColorC[i]/ColorMax; 
  }
       
 
// affichage des résultats en console etc.  (autre variante pour affichage LCD )
  Serial.print("Bleu : "); Serial.print(ColorN[0]);
  Serial.print("  Vert : "); Serial.print(ColorN[1]);
  Serial.print("  Rouge : "); Serial.println(ColorN[2]);
  Serial.println("");
  
  delay (2000)  // pause
}// Fin de loop


// Variante   Choix du max avec des if

   if(ColorC[0]<=ColorC[1]){  // test logique  <=  fig12 p 17
      if(ColorC[1]<=ColorC[2])
         {ColorMax=ColorC[2];}
      else
         {ColorMax=ColorC[1];}  // affectation de valeur
   }
   else  {
      if(ColorC[0]<=ColorC[2])
         {ColorMax=ColorC[2];}
      else
         {ColorMax=ColorC[0];}
   }
