/**************************************************************************
  LUXMETER   Mesure instantanée et  moyenne glissée sur nMax valeur) 
  fonction pour la moyenne et procédure affichage sur OLED
  
  Example of BH1750 library usage.
 
  Connection:
    VCC -> 3V3 
    GND -> GND
    SCL -> SCL (A5 on Arduino Uno, nano, etc or 21 on Mega and Due)
    SDA -> SDA (A4 on Arduino Uno, nano, etc or 20 on Mega and Due)
    ADD -> GND  sensor address 0x23 (by default in the librairy
   
**************************************************************************/
// Capteur BH1750  I2C 0x23
#include <Wire.h>
#include <BH1750.h>
BH1750 luxMeter;  // Déclaration d'alias

// Declaration pour Ecran OLED 128*32 pixels Driver SSD1306 I2C 0x3c
#include <SPI.h>
//#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// Variables globales *****************************************************************************************
const byte nMax = 10;  // nombre de mesures pour la moyenne glissée sur les nMax dernières mesures
int tabLux[nMax]; // pour le stockage des nMax dernières valeurs  
byte n; // indice du tableau de valeurs tabLux[n], donc compteur cyclique de la boucle principale
int luxMoyen; // pour la boucle For de calcul de la moyenne

// Fonctions et procédures ************************************************************************************
// Calcul de la moyenne glissée sur les nMax dernières valeurs du tableau local _tabLux
int luxMoyenne(byte _nMax, int _tabLux[]) 
{
    int _cumulLux = 0;
    for (byte i=0; i<nMax;i=i+1)                 // on calcule la moyenne des nMax dernières valeurs mesurées
    {                                           // conservées dans le tableau transmis en paramètre : _tabLux[]
       _cumulLux=_cumulLux+_tabLux[i];         // nMax est déclaré comme variable globale donc connu dans la fonction
    }
    return round(1.0*_cumulLux/_nMax);       // on veut une vraie division "réelle" de la valeur cumulée
}                                           // mais un arrondi à la valeur entière la plus proche

void displayResult(int _tabLux , int _luxMoyen) // _tabLux sera en fait un des éléments du tableau tabLux[]
{                                              // procédure d'affichage sur le LCD deux lignes
    display.clearDisplay();
    display.setTextColor(WHITE);        
    display.setTextSize(1);             
    display.setCursor(0,0);
    display.print("  Eclairement "); 
    display.setCursor(0,12); 
    display.print("  E(t) : "); display.print(_tabLux); display.print(" lx");
    display.setCursor(0,24); 
    display.print("  E moyen : ");display.print(_luxMoyen); display.print(" lx");
    display.display();
    delay(500);
}

//****************************************************************
void setup() 
{
  Serial.begin(9600); // Initialisation liaison série avec la console PC

  Wire.begin(); // Initialisation du  bus I2C relié au capteur
  luxMeter.begin(); // Initialisation de la librairie BH1750

// Test de bon fonctionnement du capteur; renvoie d'un booleen témoin du bon fonctionnement.
  if (luxMeter.begin(BH1750::CONTINUOUS_HIGH_RES_MODE_2)) 
       { Serial.println("BH1750 High Resolution mode 2"); }
  else { Serial.println("Error initialising BH1750");   }

// SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32
       Serial.println(F("SSD1306 allocation failed"));
      for(;;); // Don't proceed, loop forever
      }

//  
// Initialisationu tableau tabLux à des valeurs nulles
   for (byte i=0; i<nMax; i=i+1) //
     {tabLux[i]=0;}
// Initialisation du compteur de boucle n destiné à parcourir successivement 
// les différentes valeurs du tableau
   n=0;    
      
   Serial.println("setup done");
}// Fin de setup()
//****************************************************************************


void loop() 
{
// Mise à jour cyclique des nMax valeurs du tableaux 
    tabLux[n] = luxMeter.readLightLevel();
    Serial.print("tabLux[");  Serial.print(n);Serial.print("] : ");
    Serial.print(tabLux[n]); Serial.println(" lx");

// Calcul de la moyenne glissée sur les nMax dernières valeurs du tableau tabLux
    luxMoyen=luxMoyenne(nMax, tabLux); 
    Serial.print("luxMoyen : ");  Serial.println(luxMoyen);
   
// Affichage OLED sur deux lignes de la valeur instantanée et de la moyenne glissée
   displayResult(tabLux[n], luxMoyen);
     
// Gestion du compteur de boucle principale    
    n=n+1;// pour élément suivant
    if (n==nMax){ n=0;}   
    // après le nMaxième, on rafraichit le premier élément du tableau etc.
   // dans le test logique : égalité numérique == et pas affectation de valeur = 
        
        
}  // Fin de Loop()
//******************************************************************************************************************
//******************************************************************************************************************










*/

//Serial.print(n);Serial.print("   lux : ");Serial.println(lux); 
//Serial.print("moyenne tabLux[");  Serial.print(i);Serial.print("] : ");Serial.println(tabLux[i]);
//Serial.print("cumulLux : ");  Serial.println(cumulLux);
